
# Python imports
import sys
import os

# CORBA imports
from omniORB import CORBA, PortableServer, BiDirPolicy
import CosNaming

# IDL imports
import cb, cb__POA

use_bidir = False

orb = None

class RootFinderImpl(cb__POA.RootFinder):
    def __init__(self, tc_root):
        self.tc_root = tc_root

    def get(self):
        print "Returning CallbackServer reference"
        return self.tc_root

class CallbackServerImpl(cb__POA.CallbackServer):
    def __init__(self):
        pass

    def one_time(self, callback, message):
        print "Calling back with:", message
        callback.call_back(message)
        print "Callbacked with:", message

def run_server():

    args = sys.argv[:]

    uri = "giop:tcp::5024"
    args += ["-ORBendPoint", uri]

    global use_bidir
    if "-b" in args:
        use_bidir = True
        
    if use_bidir:
        args.append("-ORBofferBiDirectionalGIOP")
        args.append("1")
        args.append("-ORBacceptBiDirectionalGIOP")
        args.append("1")
        args.append("-ORBserverTransportRule")
        args.append("* tcp,unix,ssl,bidir")
        args.append("-ORBclientTransportRule")
        args.append("* unix,tcp,ssl,bidir")

    # Initialise the ORB
    global orb
    orb = CORBA.ORB_init(args, CORBA.ORB_ID)

    # Find the root POA
    poa = orb.resolve_initial_references("RootPOA")
    poa_man = poa._get_the_POAManager()
    poa_man.activate()

    poa_ins = orb.resolve_initial_references("omniINSPOA")
    pman_ins = poa_ins._get_the_POAManager()
    pman_ins.activate()

    # create main server servant
    server_i = CallbackServerImpl()

    # activate the it with a POA
    if use_bidir:
        # create a BiDir POA and activate it with that
        policies = [ BiDirPolicy.BidirectionalPolicy(BiDirPolicy.BOTH) ]
        bidir_poa = poa.create_POA("BidDirPOA", poa_man, policies)
        bidir_poaManager = bidir_poa._get_the_POAManager()
        bidir_poaManager.activate()
        id = bidir_poa.activate_object(server_i)

    server_ref = server_i._this()

    root_finder = RootFinderImpl(server_ref)
    poa_ins.activate_object_with_id("CallbackServerRoot", root_finder)

    # Now run the orb
    orb.run()



if __name__ == '__main__':
    run_server()
